{% extends "base.html" %}

{% block title %}Log out{% endblock %}

{% block content_title %}Log out{% endblock %}

{% block content %}
  <p>Click the button below to log out of the site.</p>
  <form action="{{ url_for('logout') }}" class="form-horizontal" method="post">
    <button class="btn btn-primary" type="submit">Log out</button>
  </form>
{% endblock %}
